#!/usr/bin/python
# -*- coding: utf-8 -*-


from abc import ABCMeta, abstractmethod
from enum import Enum
from typing import List, Tuple
from sqlalchemy.sql import exists

from .utils import DruckerConfig, PredictLabel, PredictResult, EvaluateResult, EvaluateDetail
from .logger import logger
from .models import db, ModelAssignment


class Drucker(metaclass=ABCMeta):
    __type_input = None
    __type_output = None
    config = None
    logger = None
    db = None
    predictor = None
    model_path = None

    def __init__(self, config_file: str = None):
        self.config = DruckerConfig(config_file)
        self.logger = logger
        self.db = db
        self.logger.init_app(self.config)
        self.db.init_app(self.config)

        if self.config.TEST_MODE:
            self.db.ModelBase.metadata.drop_all(self.db.engine)
        self.db.ModelBase.metadata.create_all(self.db.engine)

        if not self.db.session.query(exists().where(ModelAssignment.service_name == self.config.SERVICE_NAME)).scalar():
            model_assignment = ModelAssignment()
            model_assignment.service_name = self.config.SERVICE_NAME
            model_assignment.model_path = self.config.FILE_MODEL
            model_assignment.first_boot = True
            self.db.session.add(model_assignment)
            self.db.session.commit()

        self.model_path = self.get_model_path()

    def is_first_boot(self) -> bool:
        try:
            model_assignment = self.db.session.query(ModelAssignment).filter(ModelAssignment.service_name == self.config.SERVICE_NAME).one()
            return model_assignment.first_boot
        except:
            return True

    def get_model_path(self, model_path: str = None) -> str:
        if model_path is None:
            model_path = self.config.FILE_MODEL
            result = self.db.session.query(ModelAssignment). \
                filter(ModelAssignment.service_name == self.config.SERVICE_NAME). \
                one_or_none()
            if result is not None:
                model_path = result.model_path
        return "{0}/{1}/{2}".format(self.config.DIR_MODEL, self.config.APPLICATION_NAME, model_path)

    def get_eval_path(self, eval_path: str) -> str:
        return "{0}/{1}/{2}".format(self.config.DIR_EVAL, self.config.APPLICATION_NAME, eval_path)

    def set_type(self, type_input: Enum, type_output: Enum) -> None:
        self.__type_input = type_input
        self.__type_output = type_output

    def get_type_input(self) -> Enum:
        return self.__type_input

    def get_type_output(self) -> Enum:
        return self.__type_output

    @abstractmethod
    def load_model(self) -> None:
        raise NotImplemented()

    @abstractmethod
    def predict(self, input: PredictLabel, option: dict = None) -> PredictResult:
        raise NotImplemented()

    @abstractmethod
    def evaluate(self, file_path: str) -> Tuple[EvaluateResult, List[EvaluateDetail]]:
        raise NotImplemented()



# coding: utf-8


import os
import yaml
import json

from enum import Enum
from typing import Union, List, Dict, NamedTuple


PredictLabel = Union[str, bytes, List[str], List[int], List[float]]
PredictScore = Union[float, List[float]]


class DruckerConfig:
    def __init__(self, config_file: str):
        settings_yaml = os.getenv("DRUCKER_SETTINGS_YAML", config_file)
        config = dict()
        if settings_yaml is not None:
            config = yaml.load(open(settings_yaml, 'r'))
        self.TEST_MODE = str(os.getenv("DRUCKER_TEST_MODE", config.get("test", "False"))).lower() == 'true'
        self.SERVICE_PORT = os.getenv("DRUCKER_SERVICE_PORT", config.get("app.port", "5000"))
        self.APPLICATION_NAME = os.getenv("DRUCKER_APPLICATION_NAME", config["app.name"])
        self.SERVICE_NAME = os.getenv("DRUCKER_SERVICE_NAME", config["app.service.name"])
        service_level = os.getenv("DRUCKER_SERVICE_LEVEL", config["app.service.level"])
        self.SERVICE_LEVEL_ENUM = ServiceEnvType.to_Enum(service_level)
        self.SERVICE_INFRA = os.getenv("DRUCKER_SERVICE_INFRA", "default")
        self.DIR_MODEL = os.getenv("DRUCKER_SERVICE_MODEL_DIR", config.get("app.modeldir", "./model"))
        self.DIR_EVAL = os.getenv("DRUCKER_SERVICE_EVAL_DIR", config.get("app.evaldir", "./eval"))
        self.FILE_MODEL = os.getenv("DRUCKER_SERVICE_MODEL_FILE", config.get("app.modelfile", "default.model"))
        self.DB_MODE = os.getenv('DRUCKER_DB_MODE', config.get('use.db', "sqlite"))
        self.DB_MYSQL_HOST = os.getenv('DRUCKER_DB_MYSQL_HOST', config.get('db.mysql.host', ""))
        self.DB_MYSQL_PORT = os.getenv('DRUCKER_DB_MYSQL_PORT', config.get('db.mysql.port', ""))
        self.DB_MYSQL_DBNAME = os.getenv('DRUCKER_DB_MYSQL_DBNAME', config.get('db.mysql.dbname', ""))
        self.DB_MYSQL_USER = os.getenv('DRUCKER_DB_MYSQL_USER', config.get('db.mysql.user', ""))
        self.DB_MYSQL_PASSWORD = os.getenv('DRUCKER_DB_MYSQL_PASSWORD', config.get('db.mysql.password', ""))


class ServiceEnvType(Enum):
    DEVELOPMENT = 'development'
    BETA = 'beta'
    STAGING = 'staging'
    SANDBOX = 'sandbox'
    PRODUCTION = 'production'

    @classmethod
    def to_Enum(cls, istr: str):
        if cls.DEVELOPMENT.value == istr:
            return cls.DEVELOPMENT
        elif cls.BETA.value == istr:
            return cls.BETA
        elif cls.STAGING.value == istr:
            return cls.STAGING
        elif cls.SANDBOX.value == istr:
            return cls.SANDBOX
        elif cls.PRODUCTION.value == istr:
            return cls.PRODUCTION
        else:
            raise ValueError("'{}' is not supported as ServiceEnvType".format(istr))


class PredictResult:
    def __init__(self, label: PredictLabel, score: PredictScore, option: dict = None):
        self.label = label
        self.score = score
        self.option = json.dumps(option) if option is not None else '{}'


class EvaluateResult:
    def __init__(self, num: int = None, accuracy: float = None,
                 precision: List[float] = None, recall: List[float] = None,
                 fvalue: List[float] = None, option: Dict[str, float] = {}):
        if num is None:
            self.num = 0
            self.accuracy = 0.0
            self.precision = [0.0]
            self.recall = [0.0]
            self.fvalue = [0.0]
            self.option = {}
        else:
            self.num = num
            self.accuracy = accuracy
            self.precision = precision
            self.recall = recall
            self.fvalue = fvalue
            self.option = option


class EvaluateDetail(NamedTuple):
    result: PredictResult
    is_correct: bool


incoming_headers = [
    'x-request-id', 'x-b3-traceid', 'x-b3-spanid', 'x-b3-parentspanid',
    'x-b3-sampled', 'x-b3-flags', 'x-ot-span-context']

def getForwardHeaders(incoming: list) -> list:
    headers = list()
    for k,v in incoming:
        if k in incoming_headers:
            headers.append((k, v))
    return headers



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: drucker.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='drucker.proto',
  package='drucker',
  syntax='proto3',
  serialized_pb=_b('\n\rdrucker.proto\x12\x07\x64rucker\x1a google/protobuf/descriptor.proto\"%\n\x12HealthCheckRequest\x12\x0f\n\x07service\x18\x01 \x01(\t\"\x8d\x01\n\x13HealthCheckResponse\x12:\n\x06status\x18\x01 \x01(\x0e\x32*.drucker.HealthCheckResponse.ServingStatus\":\n\rServingStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x12\x0f\n\x0bNOT_SERVING\x10\x02\"\x18\n\tArrString\x12\x0b\n\x03val\x18\x01 \x03(\t\"$\n\x06Tensor\x12\r\n\x05shape\x18\x01 \x03(\x05\x12\x0b\n\x03val\x18\x02 \x03(\x01\"V\n\x02IO\x12!\n\x06tensor\x18\x01 \x01(\x0b\x32\x0f.drucker.TensorH\x00\x12!\n\x03str\x18\x02 \x01(\x0b\x32\x12.drucker.ArrStringH\x00\x42\n\n\x08io_oneof\"\x14\n\x12ServiceInfoRequest\"\\\n\x13ServiceInfoResponse\x12\x18\n\x10\x61pplication_name\x18\x01 \x01(\t\x12\x14\n\x0cservice_name\x18\x02 \x01(\t\x12\x15\n\rservice_level\x18\x03 \x01(\t\"0\n\x12UploadModelRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"\"\n\x12SwitchModelRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\"0\n\rModelResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\">\n\x1bUploadEvaluationDataRequest\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x11\n\tdata_path\x18\x02 \x01(\t\"?\n\x1cUploadEvaluationDataResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\"P\n\x14\x45valuateModelRequest\x12\x10\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x42\x02\x18\x01\x12\x11\n\tdata_path\x18\x02 \x01(\t\x12\x13\n\x0bresult_path\x18\x03 \x01(\t\"\xcc\x01\n\x11\x45valuationMetrics\x12\x0b\n\x03num\x18\x01 \x01(\x05\x12\x10\n\x08\x61\x63\x63uracy\x18\x02 \x01(\x02\x12\x11\n\tprecision\x18\x03 \x03(\x02\x12\x0e\n\x06recall\x18\x04 \x03(\x02\x12\x0e\n\x06\x66value\x18\x05 \x03(\x02\x12\x36\n\x06option\x18\x06 \x03(\x0b\x32&.drucker.EvaluationMetrics.OptionEntry\x1a-\n\x0bOptionEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\"D\n\x15\x45valuateModelResponse\x12+\n\x07metrics\x18\x01 \x01(\x0b\x32\x1a.drucker.EvaluationMetrics\",\n\x17\x45valuationResultRequest\x12\x11\n\tdata_path\x18\x01 \x01(\t\"\x84\x02\n\x18\x45valuationResultResponse\x12+\n\x07metrics\x18\x01 \x01(\x0b\x32\x1a.drucker.EvaluationMetrics\x12\x38\n\x06\x64\x65tail\x18\x02 \x03(\x0b\x32(.drucker.EvaluationResultResponse.Detail\x1a\x80\x01\n\x06\x44\x65tail\x12\x1a\n\x05input\x18\x01 \x01(\x0b\x32\x0b.drucker.IO\x12\x1a\n\x05label\x18\x02 \x01(\x0b\x32\x0b.drucker.IO\x12\x1b\n\x06output\x18\x03 \x01(\x0b\x32\x0b.drucker.IO\x12\r\n\x05score\x18\x04 \x03(\x02\x12\x12\n\nis_correct\x18\x05 \x01(\x08\"\x15\n\x06Option\x12\x0b\n\x03val\x18\x01 \x01(\t\"=\n\x0bStringInput\x12\r\n\x05input\x18\x01 \x01(\t\x12\x1f\n\x06option\x18\x02 \x01(\x0b\x32\x0f.drucker.Option\"<\n\nBytesInput\x12\r\n\x05input\x18\x01 \x01(\x0c\x12\x1f\n\x06option\x18\x02 \x01(\x0b\x32\x0f.drucker.Option\"=\n\x0b\x41rrIntInput\x12\r\n\x05input\x18\x01 \x03(\x05\x12\x1f\n\x06option\x18\x02 \x01(\x0b\x32\x0f.drucker.Option\"?\n\rArrFloatInput\x12\r\n\x05input\x18\x01 \x03(\x02\x12\x1f\n\x06option\x18\x02 \x01(\x0b\x32\x0f.drucker.Option\"@\n\x0e\x41rrStringInput\x12\r\n\x05input\x18\x01 \x03(\t\x12\x1f\n\x06option\x18\x02 \x01(\x0b\x32\x0f.drucker.Option\"N\n\x0cStringOutput\x12\x0e\n\x06output\x18\x01 \x01(\t\x12\r\n\x05score\x18\x02 \x01(\x02\x12\x1f\n\x06option\x18\x03 \x01(\x0b\x32\x0f.drucker.Option\"M\n\x0b\x42ytesOutput\x12\x0e\n\x06output\x18\x01 \x01(\x0c\x12\r\n\x05score\x18\x02 \x01(\x02\x12\x1f\n\x06option\x18\x03 \x01(\x0b\x32\x0f.drucker.Option\"N\n\x0c\x41rrIntOutput\x12\x0e\n\x06output\x18\x01 \x03(\x05\x12\r\n\x05score\x18\x02 \x03(\x02\x12\x1f\n\x06option\x18\x03 \x01(\x0b\x32\x0f.drucker.Option\"P\n\x0e\x41rrFloatOutput\x12\x0e\n\x06output\x18\x01 \x03(\x02\x12\r\n\x05score\x18\x02 \x03(\x02\x12\x1f\n\x06option\x18\x03 \x01(\x0b\x32\x0f.drucker.Option\"Q\n\x0f\x41rrStringOutput\x12\x0e\n\x06output\x18\x01 \x03(\t\x12\r\n\x05score\x18\x02 \x03(\x02\x12\x1f\n\x06option\x18\x03 \x01(\x0b\x32\x0f.drucker.Option*)\n\x0f\x45numVersionInfo\x12\x06\n\x02v0\x10\x00\x12\x06\n\x02v1\x10\x01\x12\x06\n\x02v2\x10\x02\x32L\n\x06Health\x12\x42\n\x05\x43heck\x12\x1b.drucker.HealthCheckRequest\x1a\x1c.drucker.HealthCheckResponse2\x86\x04\n\x10\x44ruckerDashboard\x12J\n\x0bServiceInfo\x12\x1b.drucker.ServiceInfoRequest\x1a\x1c.drucker.ServiceInfoResponse\"\x00\x12\x46\n\x0bUploadModel\x12\x1b.drucker.UploadModelRequest\x1a\x16.drucker.ModelResponse\"\x00(\x01\x12\x44\n\x0bSwitchModel\x12\x1b.drucker.SwitchModelRequest\x1a\x16.drucker.ModelResponse\"\x00\x12g\n\x14UploadEvaluationData\x12$.drucker.UploadEvaluationDataRequest\x1a%.drucker.UploadEvaluationDataResponse\"\x00(\x01\x12R\n\rEvaluateModel\x12\x1d.drucker.EvaluateModelRequest\x1a\x1e.drucker.EvaluateModelResponse\"\x00(\x01\x12[\n\x10\x45valuationResult\x12 .drucker.EvaluationResultRequest\x1a!.drucker.EvaluationResultResponse\"\x00\x30\x01\x32\xfb\x0e\n\rDruckerWorker\x12\x46\n\x15Predict_String_String\x12\x14.drucker.StringInput\x1a\x15.drucker.StringOutput\"\x00\x12\x46\n\x14Predict_String_Bytes\x12\x14.drucker.StringInput\x1a\x14.drucker.BytesOutput\"\x00\x30\x01\x12\x46\n\x15Predict_String_ArrInt\x12\x14.drucker.StringInput\x1a\x15.drucker.ArrIntOutput\"\x00\x12J\n\x17Predict_String_ArrFloat\x12\x14.drucker.StringInput\x1a\x17.drucker.ArrFloatOutput\"\x00\x12L\n\x18Predict_String_ArrString\x12\x14.drucker.StringInput\x1a\x18.drucker.ArrStringOutput\"\x00\x12\x46\n\x14Predict_Bytes_String\x12\x13.drucker.BytesInput\x1a\x15.drucker.StringOutput\"\x00(\x01\x12\x46\n\x13Predict_Bytes_Bytes\x12\x13.drucker.BytesInput\x1a\x14.drucker.BytesOutput\"\x00(\x01\x30\x01\x12\x46\n\x14Predict_Bytes_ArrInt\x12\x13.drucker.BytesInput\x1a\x15.drucker.ArrIntOutput\"\x00(\x01\x12J\n\x16Predict_Bytes_ArrFloat\x12\x13.drucker.BytesInput\x1a\x17.drucker.ArrFloatOutput\"\x00(\x01\x12L\n\x17Predict_Bytes_ArrString\x12\x13.drucker.BytesInput\x1a\x18.drucker.ArrStringOutput\"\x00(\x01\x12\x46\n\x15Predict_ArrInt_String\x12\x14.drucker.ArrIntInput\x1a\x15.drucker.StringOutput\"\x00\x12\x46\n\x14Predict_ArrInt_Bytes\x12\x14.drucker.ArrIntInput\x1a\x14.drucker.BytesOutput\"\x00\x30\x01\x12\x46\n\x15Predict_ArrInt_ArrInt\x12\x14.drucker.ArrIntInput\x1a\x15.drucker.ArrIntOutput\"\x00\x12J\n\x17Predict_ArrInt_ArrFloat\x12\x14.drucker.ArrIntInput\x1a\x17.drucker.ArrFloatOutput\"\x00\x12L\n\x18Predict_ArrInt_ArrString\x12\x14.drucker.ArrIntInput\x1a\x18.drucker.ArrStringOutput\"\x00\x12J\n\x17Predict_ArrFloat_String\x12\x16.drucker.ArrFloatInput\x1a\x15.drucker.StringOutput\"\x00\x12J\n\x16Predict_ArrFloat_Bytes\x12\x16.drucker.ArrFloatInput\x1a\x14.drucker.BytesOutput\"\x00\x30\x01\x12J\n\x17Predict_ArrFloat_ArrInt\x12\x16.drucker.ArrFloatInput\x1a\x15.drucker.ArrIntOutput\"\x00\x12N\n\x19Predict_ArrFloat_ArrFloat\x12\x16.drucker.ArrFloatInput\x1a\x17.drucker.ArrFloatOutput\"\x00\x12P\n\x1aPredict_ArrFloat_ArrString\x12\x16.drucker.ArrFloatInput\x1a\x18.drucker.ArrStringOutput\"\x00\x12L\n\x18Predict_ArrString_String\x12\x17.drucker.ArrStringInput\x1a\x15.drucker.StringOutput\"\x00\x12L\n\x17Predict_ArrString_Bytes\x12\x17.drucker.ArrStringInput\x1a\x14.drucker.BytesOutput\"\x00\x30\x01\x12L\n\x18Predict_ArrString_ArrInt\x12\x17.drucker.ArrStringInput\x1a\x15.drucker.ArrIntOutput\"\x00\x12P\n\x1aPredict_ArrString_ArrFloat\x12\x17.drucker.ArrStringInput\x1a\x17.drucker.ArrFloatOutput\"\x00\x12R\n\x1bPredict_ArrString_ArrString\x12\x17.drucker.ArrStringInput\x1a\x18.drucker.ArrStringOutput\"\x00:B\n\x1a\x64rucker_grpc_proto_version\x12\x1c.google.protobuf.FileOptions\x18\xd0\x86\x03 \x01(\tB\x06\x82\xb5\x18\x02v2b\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_descriptor__pb2.DESCRIPTOR,])

_ENUMVERSIONINFO = _descriptor.EnumDescriptor(
  name='EnumVersionInfo',
  full_name='drucker.EnumVersionInfo',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='v0', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='v1', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='v2', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2190,
  serialized_end=2231,
)
_sym_db.RegisterEnumDescriptor(_ENUMVERSIONINFO)

EnumVersionInfo = enum_type_wrapper.EnumTypeWrapper(_ENUMVERSIONINFO)
v0 = 0
v1 = 1
v2 = 2

DRUCKER_GRPC_PROTO_VERSION_FIELD_NUMBER = 50000
drucker_grpc_proto_version = _descriptor.FieldDescriptor(
  name='drucker_grpc_proto_version', full_name='drucker.drucker_grpc_proto_version', index=0,
  number=50000, type=9, cpp_type=9, label=1,
  has_default_value=False, default_value=_b("").decode('utf-8'),
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  options=None, file=DESCRIPTOR)

_HEALTHCHECKRESPONSE_SERVINGSTATUS = _descriptor.EnumDescriptor(
  name='ServingStatus',
  full_name='drucker.HealthCheckResponse.ServingStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SERVING', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NOT_SERVING', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=183,
  serialized_end=241,
)
_sym_db.RegisterEnumDescriptor(_HEALTHCHECKRESPONSE_SERVINGSTATUS)


_HEALTHCHECKREQUEST = _descriptor.Descriptor(
  name='HealthCheckRequest',
  full_name='drucker.HealthCheckRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='service', full_name='drucker.HealthCheckRequest.service', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=97,
)


_HEALTHCHECKRESPONSE = _descriptor.Descriptor(
  name='HealthCheckResponse',
  full_name='drucker.HealthCheckResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='drucker.HealthCheckResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _HEALTHCHECKRESPONSE_SERVINGSTATUS,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=100,
  serialized_end=241,
)


_ARRSTRING = _descriptor.Descriptor(
  name='ArrString',
  full_name='drucker.ArrString',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='val', full_name='drucker.ArrString.val', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=243,
  serialized_end=267,
)


_TENSOR = _descriptor.Descriptor(
  name='Tensor',
  full_name='drucker.Tensor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='shape', full_name='drucker.Tensor.shape', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='val', full_name='drucker.Tensor.val', index=1,
      number=2, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=269,
  serialized_end=305,
)


_IO = _descriptor.Descriptor(
  name='IO',
  full_name='drucker.IO',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tensor', full_name='drucker.IO.tensor', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='str', full_name='drucker.IO.str', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='io_oneof', full_name='drucker.IO.io_oneof',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=307,
  serialized_end=393,
)


_SERVICEINFOREQUEST = _descriptor.Descriptor(
  name='ServiceInfoRequest',
  full_name='drucker.ServiceInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=395,
  serialized_end=415,
)


_SERVICEINFORESPONSE = _descriptor.Descriptor(
  name='ServiceInfoResponse',
  full_name='drucker.ServiceInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='application_name', full_name='drucker.ServiceInfoResponse.application_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='service_name', full_name='drucker.ServiceInfoResponse.service_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='service_level', full_name='drucker.ServiceInfoResponse.service_level', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=417,
  serialized_end=509,
)


_UPLOADMODELREQUEST = _descriptor.Descriptor(
  name='UploadModelRequest',
  full_name='drucker.UploadModelRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='drucker.UploadModelRequest.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='drucker.UploadModelRequest.data', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=511,
  serialized_end=559,
)


_SWITCHMODELREQUEST = _descriptor.Descriptor(
  name='SwitchModelRequest',
  full_name='drucker.SwitchModelRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='drucker.SwitchModelRequest.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=561,
  serialized_end=595,
)


_MODELRESPONSE = _descriptor.Descriptor(
  name='ModelResponse',
  full_name='drucker.ModelResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='drucker.ModelResponse.status', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='drucker.ModelResponse.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=597,
  serialized_end=645,
)


_UPLOADEVALUATIONDATAREQUEST = _descriptor.Descriptor(
  name='UploadEvaluationDataRequest',
  full_name='drucker.UploadEvaluationDataRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='drucker.UploadEvaluationDataRequest.data', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data_path', full_name='drucker.UploadEvaluationDataRequest.data_path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=647,
  serialized_end=709,
)


_UPLOADEVALUATIONDATARESPONSE = _descriptor.Descriptor(
  name='UploadEvaluationDataResponse',
  full_name='drucker.UploadEvaluationDataResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='drucker.UploadEvaluationDataResponse.status', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='drucker.UploadEvaluationDataResponse.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=711,
  serialized_end=774,
)


_EVALUATEMODELREQUEST = _descriptor.Descriptor(
  name='EvaluateModelRequest',
  full_name='drucker.EvaluateModelRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='drucker.EvaluateModelRequest.data', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001')), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data_path', full_name='drucker.EvaluateModelRequest.data_path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='result_path', full_name='drucker.EvaluateModelRequest.result_path', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=776,
  serialized_end=856,
)


_EVALUATIONMETRICS_OPTIONENTRY = _descriptor.Descriptor(
  name='OptionEntry',
  full_name='drucker.EvaluationMetrics.OptionEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='drucker.EvaluationMetrics.OptionEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='drucker.EvaluationMetrics.OptionEntry.value', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1018,
  serialized_end=1063,
)

_EVALUATIONMETRICS = _descriptor.Descriptor(
  name='EvaluationMetrics',
  full_name='drucker.EvaluationMetrics',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='num', full_name='drucker.EvaluationMetrics.num', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='accuracy', full_name='drucker.EvaluationMetrics.accuracy', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='precision', full_name='drucker.EvaluationMetrics.precision', index=2,
      number=3, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recall', full_name='drucker.EvaluationMetrics.recall', index=3,
      number=4, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fvalue', full_name='drucker.EvaluationMetrics.fvalue', index=4,
      number=5, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='option', full_name='drucker.EvaluationMetrics.option', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_EVALUATIONMETRICS_OPTIONENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=859,
  serialized_end=1063,
)


_EVALUATEMODELRESPONSE = _descriptor.Descriptor(
  name='EvaluateModelResponse',
  full_name='drucker.EvaluateModelResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='metrics', full_name='drucker.EvaluateModelResponse.metrics', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1065,
  serialized_end=1133,
)


_EVALUATIONRESULTREQUEST = _descriptor.Descriptor(
  name='EvaluationResultRequest',
  full_name='drucker.EvaluationResultRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data_path', full_name='drucker.EvaluationResultRequest.data_path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1135,
  serialized_end=1179,
)


_EVALUATIONRESULTRESPONSE_DETAIL = _descriptor.Descriptor(
  name='Detail',
  full_name='drucker.EvaluationResultResponse.Detail',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='input', full_name='drucker.EvaluationResultResponse.Detail.input', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label', full_name='drucker.EvaluationResultResponse.Detail.label', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='output', full_name='drucker.EvaluationResultResponse.Detail.output', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='drucker.EvaluationResultResponse.Detail.score', index=3,
      number=4, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_correct', full_name='drucker.EvaluationResultResponse.Detail.is_correct', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1314,
  serialized_end=1442,
)

_EVALUATIONRESULTRESPONSE = _descriptor.Descriptor(
  name='EvaluationResultResponse',
  full_name='drucker.EvaluationResultResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='metrics', full_name='drucker.EvaluationResultResponse.metrics', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='detail', full_name='drucker.EvaluationResultResponse.detail', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_EVALUATIONRESULTRESPONSE_DETAIL, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1182,
  serialized_end=1442,
)


_OPTION = _descriptor.Descriptor(
  name='Option',
  full_name='drucker.Option',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='val', full_name='drucker.Option.val', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1444,
  serialized_end=1465,
)


_STRINGINPUT = _descriptor.Descriptor(
  name='StringInput',
  full_name='drucker.StringInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='input', full_name='drucker.StringInput.input', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='option', full_name='drucker.StringInput.option', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1467,
  serialized_end=1528,
)


_BYTESINPUT = _descriptor.Descriptor(
  name='BytesInput',
  full_name='drucker.BytesInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='input', full_name='drucker.BytesInput.input', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='option', full_name='drucker.BytesInput.option', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1530,
  serialized_end=1590,
)


_ARRINTINPUT = _descriptor.Descriptor(
  name='ArrIntInput',
  full_name='drucker.ArrIntInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='input', full_name='drucker.ArrIntInput.input', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='option', full_name='drucker.ArrIntInput.option', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1592,
  serialized_end=1653,
)


_ARRFLOATINPUT = _descriptor.Descriptor(
  name='ArrFloatInput',
  full_name='drucker.ArrFloatInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='input', full_name='drucker.ArrFloatInput.input', index=0,
      number=1, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='option', full_name='drucker.ArrFloatInput.option', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1655,
  serialized_end=1718,
)


_ARRSTRINGINPUT = _descriptor.Descriptor(
  name='ArrStringInput',
  full_name='drucker.ArrStringInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='input', full_name='drucker.ArrStringInput.input', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='option', full_name='drucker.ArrStringInput.option', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1720,
  serialized_end=1784,
)


_STRINGOUTPUT = _descriptor.Descriptor(
  name='StringOutput',
  full_name='drucker.StringOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='output', full_name='drucker.StringOutput.output', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='drucker.StringOutput.score', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='option', full_name='drucker.StringOutput.option', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1786,
  serialized_end=1864,
)


_BYTESOUTPUT = _descriptor.Descriptor(
  name='BytesOutput',
  full_name='drucker.BytesOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='output', full_name='drucker.BytesOutput.output', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='drucker.BytesOutput.score', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='option', full_name='drucker.BytesOutput.option', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1866,
  serialized_end=1943,
)


_ARRINTOUTPUT = _descriptor.Descriptor(
  name='ArrIntOutput',
  full_name='drucker.ArrIntOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='output', full_name='drucker.ArrIntOutput.output', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='drucker.ArrIntOutput.score', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='option', full_name='drucker.ArrIntOutput.option', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1945,
  serialized_end=2023,
)


_ARRFLOATOUTPUT = _descriptor.Descriptor(
  name='ArrFloatOutput',
  full_name='drucker.ArrFloatOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='output', full_name='drucker.ArrFloatOutput.output', index=0,
      number=1, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='drucker.ArrFloatOutput.score', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='option', full_name='drucker.ArrFloatOutput.option', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2025,
  serialized_end=2105,
)


_ARRSTRINGOUTPUT = _descriptor.Descriptor(
  name='ArrStringOutput',
  full_name='drucker.ArrStringOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='output', full_name='drucker.ArrStringOutput.output', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='drucker.ArrStringOutput.score', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='option', full_name='drucker.ArrStringOutput.option', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2107,
  serialized_end=2188,
)

_HEALTHCHECKRESPONSE.fields_by_name['status'].enum_type = _HEALTHCHECKRESPONSE_SERVINGSTATUS
_HEALTHCHECKRESPONSE_SERVINGSTATUS.containing_type = _HEALTHCHECKRESPONSE
_IO.fields_by_name['tensor'].message_type = _TENSOR
_IO.fields_by_name['str'].message_type = _ARRSTRING
_IO.oneofs_by_name['io_oneof'].fields.append(
  _IO.fields_by_name['tensor'])
_IO.fields_by_name['tensor'].containing_oneof = _IO.oneofs_by_name['io_oneof']
_IO.oneofs_by_name['io_oneof'].fields.append(
  _IO.fields_by_name['str'])
_IO.fields_by_name['str'].containing_oneof = _IO.oneofs_by_name['io_oneof']
_EVALUATIONMETRICS_OPTIONENTRY.containing_type = _EVALUATIONMETRICS
_EVALUATIONMETRICS.fields_by_name['option'].message_type = _EVALUATIONMETRICS_OPTIONENTRY
_EVALUATEMODELRESPONSE.fields_by_name['metrics'].message_type = _EVALUATIONMETRICS
_EVALUATIONRESULTRESPONSE_DETAIL.fields_by_name['input'].message_type = _IO
_EVALUATIONRESULTRESPONSE_DETAIL.fields_by_name['label'].message_type = _IO
_EVALUATIONRESULTRESPONSE_DETAIL.fields_by_name['output'].message_type = _IO
_EVALUATIONRESULTRESPONSE_DETAIL.containing_type = _EVALUATIONRESULTRESPONSE
_EVALUATIONRESULTRESPONSE.fields_by_name['metrics'].message_type = _EVALUATIONMETRICS
_EVALUATIONRESULTRESPONSE.fields_by_name['detail'].message_type = _EVALUATIONRESULTRESPONSE_DETAIL
_STRINGINPUT.fields_by_name['option'].message_type = _OPTION
_BYTESINPUT.fields_by_name['option'].message_type = _OPTION
_ARRINTINPUT.fields_by_name['option'].message_type = _OPTION
_ARRFLOATINPUT.fields_by_name['option'].message_type = _OPTION
_ARRSTRINGINPUT.fields_by_name['option'].message_type = _OPTION
_STRINGOUTPUT.fields_by_name['option'].message_type = _OPTION
_BYTESOUTPUT.fields_by_name['option'].message_type = _OPTION
_ARRINTOUTPUT.fields_by_name['option'].message_type = _OPTION
_ARRFLOATOUTPUT.fields_by_name['option'].message_type = _OPTION
_ARRSTRINGOUTPUT.fields_by_name['option'].message_type = _OPTION
DESCRIPTOR.message_types_by_name['HealthCheckRequest'] = _HEALTHCHECKREQUEST
DESCRIPTOR.message_types_by_name['HealthCheckResponse'] = _HEALTHCHECKRESPONSE
DESCRIPTOR.message_types_by_name['ArrString'] = _ARRSTRING
DESCRIPTOR.message_types_by_name['Tensor'] = _TENSOR
DESCRIPTOR.message_types_by_name['IO'] = _IO
DESCRIPTOR.message_types_by_name['ServiceInfoRequest'] = _SERVICEINFOREQUEST
DESCRIPTOR.message_types_by_name['ServiceInfoResponse'] = _SERVICEINFORESPONSE
DESCRIPTOR.message_types_by_name['UploadModelRequest'] = _UPLOADMODELREQUEST
DESCRIPTOR.message_types_by_name['SwitchModelRequest'] = _SWITCHMODELREQUEST
DESCRIPTOR.message_types_by_name['ModelResponse'] = _MODELRESPONSE
DESCRIPTOR.message_types_by_name['UploadEvaluationDataRequest'] = _UPLOADEVALUATIONDATAREQUEST
DESCRIPTOR.message_types_by_name['UploadEvaluationDataResponse'] = _UPLOADEVALUATIONDATARESPONSE
DESCRIPTOR.message_types_by_name['EvaluateModelRequest'] = _EVALUATEMODELREQUEST
DESCRIPTOR.message_types_by_name['EvaluationMetrics'] = _EVALUATIONMETRICS
DESCRIPTOR.message_types_by_name['EvaluateModelResponse'] = _EVALUATEMODELRESPONSE
DESCRIPTOR.message_types_by_name['EvaluationResultRequest'] = _EVALUATIONRESULTREQUEST
DESCRIPTOR.message_types_by_name['EvaluationResultResponse'] = _EVALUATIONRESULTRESPONSE
DESCRIPTOR.message_types_by_name['Option'] = _OPTION
DESCRIPTOR.message_types_by_name['StringInput'] = _STRINGINPUT
DESCRIPTOR.message_types_by_name['BytesInput'] = _BYTESINPUT
DESCRIPTOR.message_types_by_name['ArrIntInput'] = _ARRINTINPUT
DESCRIPTOR.message_types_by_name['ArrFloatInput'] = _ARRFLOATINPUT
DESCRIPTOR.message_types_by_name['ArrStringInput'] = _ARRSTRINGINPUT
DESCRIPTOR.message_types_by_name['StringOutput'] = _STRINGOUTPUT
DESCRIPTOR.message_types_by_name['BytesOutput'] = _BYTESOUTPUT
DESCRIPTOR.message_types_by_name['ArrIntOutput'] = _ARRINTOUTPUT
DESCRIPTOR.message_types_by_name['ArrFloatOutput'] = _ARRFLOATOUTPUT
DESCRIPTOR.message_types_by_name['ArrStringOutput'] = _ARRSTRINGOUTPUT
DESCRIPTOR.enum_types_by_name['EnumVersionInfo'] = _ENUMVERSIONINFO
DESCRIPTOR.extensions_by_name['drucker_grpc_proto_version'] = drucker_grpc_proto_version
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

HealthCheckRequest = _reflection.GeneratedProtocolMessageType('HealthCheckRequest', (_message.Message,), dict(
  DESCRIPTOR = _HEALTHCHECKREQUEST,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.HealthCheckRequest)
  ))
_sym_db.RegisterMessage(HealthCheckRequest)

HealthCheckResponse = _reflection.GeneratedProtocolMessageType('HealthCheckResponse', (_message.Message,), dict(
  DESCRIPTOR = _HEALTHCHECKRESPONSE,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.HealthCheckResponse)
  ))
_sym_db.RegisterMessage(HealthCheckResponse)

ArrString = _reflection.GeneratedProtocolMessageType('ArrString', (_message.Message,), dict(
  DESCRIPTOR = _ARRSTRING,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.ArrString)
  ))
_sym_db.RegisterMessage(ArrString)

Tensor = _reflection.GeneratedProtocolMessageType('Tensor', (_message.Message,), dict(
  DESCRIPTOR = _TENSOR,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.Tensor)
  ))
_sym_db.RegisterMessage(Tensor)

IO = _reflection.GeneratedProtocolMessageType('IO', (_message.Message,), dict(
  DESCRIPTOR = _IO,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.IO)
  ))
_sym_db.RegisterMessage(IO)

ServiceInfoRequest = _reflection.GeneratedProtocolMessageType('ServiceInfoRequest', (_message.Message,), dict(
  DESCRIPTOR = _SERVICEINFOREQUEST,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.ServiceInfoRequest)
  ))
_sym_db.RegisterMessage(ServiceInfoRequest)

ServiceInfoResponse = _reflection.GeneratedProtocolMessageType('ServiceInfoResponse', (_message.Message,), dict(
  DESCRIPTOR = _SERVICEINFORESPONSE,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.ServiceInfoResponse)
  ))
_sym_db.RegisterMessage(ServiceInfoResponse)

UploadModelRequest = _reflection.GeneratedProtocolMessageType('UploadModelRequest', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADMODELREQUEST,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.UploadModelRequest)
  ))
_sym_db.RegisterMessage(UploadModelRequest)

SwitchModelRequest = _reflection.GeneratedProtocolMessageType('SwitchModelRequest', (_message.Message,), dict(
  DESCRIPTOR = _SWITCHMODELREQUEST,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.SwitchModelRequest)
  ))
_sym_db.RegisterMessage(SwitchModelRequest)

ModelResponse = _reflection.GeneratedProtocolMessageType('ModelResponse', (_message.Message,), dict(
  DESCRIPTOR = _MODELRESPONSE,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.ModelResponse)
  ))
_sym_db.RegisterMessage(ModelResponse)

UploadEvaluationDataRequest = _reflection.GeneratedProtocolMessageType('UploadEvaluationDataRequest', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADEVALUATIONDATAREQUEST,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.UploadEvaluationDataRequest)
  ))
_sym_db.RegisterMessage(UploadEvaluationDataRequest)

UploadEvaluationDataResponse = _reflection.GeneratedProtocolMessageType('UploadEvaluationDataResponse', (_message.Message,), dict(
  DESCRIPTOR = _UPLOADEVALUATIONDATARESPONSE,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.UploadEvaluationDataResponse)
  ))
_sym_db.RegisterMessage(UploadEvaluationDataResponse)

EvaluateModelRequest = _reflection.GeneratedProtocolMessageType('EvaluateModelRequest', (_message.Message,), dict(
  DESCRIPTOR = _EVALUATEMODELREQUEST,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.EvaluateModelRequest)
  ))
_sym_db.RegisterMessage(EvaluateModelRequest)

EvaluationMetrics = _reflection.GeneratedProtocolMessageType('EvaluationMetrics', (_message.Message,), dict(

  OptionEntry = _reflection.GeneratedProtocolMessageType('OptionEntry', (_message.Message,), dict(
    DESCRIPTOR = _EVALUATIONMETRICS_OPTIONENTRY,
    __module__ = 'drucker_pb2'
    # @@protoc_insertion_point(class_scope:drucker.EvaluationMetrics.OptionEntry)
    ))
  ,
  DESCRIPTOR = _EVALUATIONMETRICS,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.EvaluationMetrics)
  ))
_sym_db.RegisterMessage(EvaluationMetrics)
_sym_db.RegisterMessage(EvaluationMetrics.OptionEntry)

EvaluateModelResponse = _reflection.GeneratedProtocolMessageType('EvaluateModelResponse', (_message.Message,), dict(
  DESCRIPTOR = _EVALUATEMODELRESPONSE,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.EvaluateModelResponse)
  ))
_sym_db.RegisterMessage(EvaluateModelResponse)

EvaluationResultRequest = _reflection.GeneratedProtocolMessageType('EvaluationResultRequest', (_message.Message,), dict(
  DESCRIPTOR = _EVALUATIONRESULTREQUEST,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.EvaluationResultRequest)
  ))
_sym_db.RegisterMessage(EvaluationResultRequest)

EvaluationResultResponse = _reflection.GeneratedProtocolMessageType('EvaluationResultResponse', (_message.Message,), dict(

  Detail = _reflection.GeneratedProtocolMessageType('Detail', (_message.Message,), dict(
    DESCRIPTOR = _EVALUATIONRESULTRESPONSE_DETAIL,
    __module__ = 'drucker_pb2'
    # @@protoc_insertion_point(class_scope:drucker.EvaluationResultResponse.Detail)
    ))
  ,
  DESCRIPTOR = _EVALUATIONRESULTRESPONSE,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.EvaluationResultResponse)
  ))
_sym_db.RegisterMessage(EvaluationResultResponse)
_sym_db.RegisterMessage(EvaluationResultResponse.Detail)

Option = _reflection.GeneratedProtocolMessageType('Option', (_message.Message,), dict(
  DESCRIPTOR = _OPTION,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.Option)
  ))
_sym_db.RegisterMessage(Option)

StringInput = _reflection.GeneratedProtocolMessageType('StringInput', (_message.Message,), dict(
  DESCRIPTOR = _STRINGINPUT,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.StringInput)
  ))
_sym_db.RegisterMessage(StringInput)

BytesInput = _reflection.GeneratedProtocolMessageType('BytesInput', (_message.Message,), dict(
  DESCRIPTOR = _BYTESINPUT,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.BytesInput)
  ))
_sym_db.RegisterMessage(BytesInput)

ArrIntInput = _reflection.GeneratedProtocolMessageType('ArrIntInput', (_message.Message,), dict(
  DESCRIPTOR = _ARRINTINPUT,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.ArrIntInput)
  ))
_sym_db.RegisterMessage(ArrIntInput)

ArrFloatInput = _reflection.GeneratedProtocolMessageType('ArrFloatInput', (_message.Message,), dict(
  DESCRIPTOR = _ARRFLOATINPUT,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.ArrFloatInput)
  ))
_sym_db.RegisterMessage(ArrFloatInput)

ArrStringInput = _reflection.GeneratedProtocolMessageType('ArrStringInput', (_message.Message,), dict(
  DESCRIPTOR = _ARRSTRINGINPUT,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.ArrStringInput)
  ))
_sym_db.RegisterMessage(ArrStringInput)

StringOutput = _reflection.GeneratedProtocolMessageType('StringOutput', (_message.Message,), dict(
  DESCRIPTOR = _STRINGOUTPUT,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.StringOutput)
  ))
_sym_db.RegisterMessage(StringOutput)

BytesOutput = _reflection.GeneratedProtocolMessageType('BytesOutput', (_message.Message,), dict(
  DESCRIPTOR = _BYTESOUTPUT,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.BytesOutput)
  ))
_sym_db.RegisterMessage(BytesOutput)

ArrIntOutput = _reflection.GeneratedProtocolMessageType('ArrIntOutput', (_message.Message,), dict(
  DESCRIPTOR = _ARRINTOUTPUT,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.ArrIntOutput)
  ))
_sym_db.RegisterMessage(ArrIntOutput)

ArrFloatOutput = _reflection.GeneratedProtocolMessageType('ArrFloatOutput', (_message.Message,), dict(
  DESCRIPTOR = _ARRFLOATOUTPUT,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.ArrFloatOutput)
  ))
_sym_db.RegisterMessage(ArrFloatOutput)

ArrStringOutput = _reflection.GeneratedProtocolMessageType('ArrStringOutput', (_message.Message,), dict(
  DESCRIPTOR = _ARRSTRINGOUTPUT,
  __module__ = 'drucker_pb2'
  # @@protoc_insertion_point(class_scope:drucker.ArrStringOutput)
  ))
_sym_db.RegisterMessage(ArrStringOutput)

google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(drucker_grpc_proto_version)

DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\202\265\030\002v2'))
_EVALUATEMODELREQUEST.fields_by_name['data'].has_options = True
_EVALUATEMODELREQUEST.fields_by_name['data']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_EVALUATIONMETRICS_OPTIONENTRY.has_options = True
_EVALUATIONMETRICS_OPTIONENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))

_HEALTH = _descriptor.ServiceDescriptor(
  name='Health',
  full_name='drucker.Health',
  file=DESCRIPTOR,
  index=0,
  options=None,
  serialized_start=2233,
  serialized_end=2309,
  methods=[
  _descriptor.MethodDescriptor(
    name='Check',
    full_name='drucker.Health.Check',
    index=0,
    containing_service=None,
    input_type=_HEALTHCHECKREQUEST,
    output_type=_HEALTHCHECKRESPONSE,
    options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_HEALTH)

DESCRIPTOR.services_by_name['Health'] = _HEALTH


_DRUCKERDASHBOARD = _descriptor.ServiceDescriptor(
  name='DruckerDashboard',
  full_name='drucker.DruckerDashboard',
  file=DESCRIPTOR,
  index=1,
  options=None,
  serialized_start=2312,
  serialized_end=2830,
  methods=[
  _descriptor.MethodDescriptor(
    name='ServiceInfo',
    full_name='drucker.DruckerDashboard.ServiceInfo',
    index=0,
    containing_service=None,
    input_type=_SERVICEINFOREQUEST,
    output_type=_SERVICEINFORESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UploadModel',
    full_name='drucker.DruckerDashboard.UploadModel',
    index=1,
    containing_service=None,
    input_type=_UPLOADMODELREQUEST,
    output_type=_MODELRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SwitchModel',
    full_name='drucker.DruckerDashboard.SwitchModel',
    index=2,
    containing_service=None,
    input_type=_SWITCHMODELREQUEST,
    output_type=_MODELRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UploadEvaluationData',
    full_name='drucker.DruckerDashboard.UploadEvaluationData',
    index=3,
    containing_service=None,
    input_type=_UPLOADEVALUATIONDATAREQUEST,
    output_type=_UPLOADEVALUATIONDATARESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='EvaluateModel',
    full_name='drucker.DruckerDashboard.EvaluateModel',
    index=4,
    containing_service=None,
    input_type=_EVALUATEMODELREQUEST,
    output_type=_EVALUATEMODELRESPONSE,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='EvaluationResult',
    full_name='drucker.DruckerDashboard.EvaluationResult',
    index=5,
    containing_service=None,
    input_type=_EVALUATIONRESULTREQUEST,
    output_type=_EVALUATIONRESULTRESPONSE,
    options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_DRUCKERDASHBOARD)

DESCRIPTOR.services_by_name['DruckerDashboard'] = _DRUCKERDASHBOARD


_DRUCKERWORKER = _descriptor.ServiceDescriptor(
  name='DruckerWorker',
  full_name='drucker.DruckerWorker',
  file=DESCRIPTOR,
  index=2,
  options=None,
  serialized_start=2833,
  serialized_end=4748,
  methods=[
  _descriptor.MethodDescriptor(
    name='Predict_String_String',
    full_name='drucker.DruckerWorker.Predict_String_String',
    index=0,
    containing_service=None,
    input_type=_STRINGINPUT,
    output_type=_STRINGOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_String_Bytes',
    full_name='drucker.DruckerWorker.Predict_String_Bytes',
    index=1,
    containing_service=None,
    input_type=_STRINGINPUT,
    output_type=_BYTESOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_String_ArrInt',
    full_name='drucker.DruckerWorker.Predict_String_ArrInt',
    index=2,
    containing_service=None,
    input_type=_STRINGINPUT,
    output_type=_ARRINTOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_String_ArrFloat',
    full_name='drucker.DruckerWorker.Predict_String_ArrFloat',
    index=3,
    containing_service=None,
    input_type=_STRINGINPUT,
    output_type=_ARRFLOATOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_String_ArrString',
    full_name='drucker.DruckerWorker.Predict_String_ArrString',
    index=4,
    containing_service=None,
    input_type=_STRINGINPUT,
    output_type=_ARRSTRINGOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_Bytes_String',
    full_name='drucker.DruckerWorker.Predict_Bytes_String',
    index=5,
    containing_service=None,
    input_type=_BYTESINPUT,
    output_type=_STRINGOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_Bytes_Bytes',
    full_name='drucker.DruckerWorker.Predict_Bytes_Bytes',
    index=6,
    containing_service=None,
    input_type=_BYTESINPUT,
    output_type=_BYTESOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_Bytes_ArrInt',
    full_name='drucker.DruckerWorker.Predict_Bytes_ArrInt',
    index=7,
    containing_service=None,
    input_type=_BYTESINPUT,
    output_type=_ARRINTOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_Bytes_ArrFloat',
    full_name='drucker.DruckerWorker.Predict_Bytes_ArrFloat',
    index=8,
    containing_service=None,
    input_type=_BYTESINPUT,
    output_type=_ARRFLOATOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_Bytes_ArrString',
    full_name='drucker.DruckerWorker.Predict_Bytes_ArrString',
    index=9,
    containing_service=None,
    input_type=_BYTESINPUT,
    output_type=_ARRSTRINGOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrInt_String',
    full_name='drucker.DruckerWorker.Predict_ArrInt_String',
    index=10,
    containing_service=None,
    input_type=_ARRINTINPUT,
    output_type=_STRINGOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrInt_Bytes',
    full_name='drucker.DruckerWorker.Predict_ArrInt_Bytes',
    index=11,
    containing_service=None,
    input_type=_ARRINTINPUT,
    output_type=_BYTESOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrInt_ArrInt',
    full_name='drucker.DruckerWorker.Predict_ArrInt_ArrInt',
    index=12,
    containing_service=None,
    input_type=_ARRINTINPUT,
    output_type=_ARRINTOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrInt_ArrFloat',
    full_name='drucker.DruckerWorker.Predict_ArrInt_ArrFloat',
    index=13,
    containing_service=None,
    input_type=_ARRINTINPUT,
    output_type=_ARRFLOATOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrInt_ArrString',
    full_name='drucker.DruckerWorker.Predict_ArrInt_ArrString',
    index=14,
    containing_service=None,
    input_type=_ARRINTINPUT,
    output_type=_ARRSTRINGOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrFloat_String',
    full_name='drucker.DruckerWorker.Predict_ArrFloat_String',
    index=15,
    containing_service=None,
    input_type=_ARRFLOATINPUT,
    output_type=_STRINGOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrFloat_Bytes',
    full_name='drucker.DruckerWorker.Predict_ArrFloat_Bytes',
    index=16,
    containing_service=None,
    input_type=_ARRFLOATINPUT,
    output_type=_BYTESOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrFloat_ArrInt',
    full_name='drucker.DruckerWorker.Predict_ArrFloat_ArrInt',
    index=17,
    containing_service=None,
    input_type=_ARRFLOATINPUT,
    output_type=_ARRINTOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrFloat_ArrFloat',
    full_name='drucker.DruckerWorker.Predict_ArrFloat_ArrFloat',
    index=18,
    containing_service=None,
    input_type=_ARRFLOATINPUT,
    output_type=_ARRFLOATOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrFloat_ArrString',
    full_name='drucker.DruckerWorker.Predict_ArrFloat_ArrString',
    index=19,
    containing_service=None,
    input_type=_ARRFLOATINPUT,
    output_type=_ARRSTRINGOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrString_String',
    full_name='drucker.DruckerWorker.Predict_ArrString_String',
    index=20,
    containing_service=None,
    input_type=_ARRSTRINGINPUT,
    output_type=_STRINGOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrString_Bytes',
    full_name='drucker.DruckerWorker.Predict_ArrString_Bytes',
    index=21,
    containing_service=None,
    input_type=_ARRSTRINGINPUT,
    output_type=_BYTESOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrString_ArrInt',
    full_name='drucker.DruckerWorker.Predict_ArrString_ArrInt',
    index=22,
    containing_service=None,
    input_type=_ARRSTRINGINPUT,
    output_type=_ARRINTOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrString_ArrFloat',
    full_name='drucker.DruckerWorker.Predict_ArrString_ArrFloat',
    index=23,
    containing_service=None,
    input_type=_ARRSTRINGINPUT,
    output_type=_ARRFLOATOUTPUT,
    options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Predict_ArrString_ArrString',
    full_name='drucker.DruckerWorker.Predict_ArrString_ArrString',
    index=24,
    containing_service=None,
    input_type=_ARRSTRINGINPUT,
    output_type=_ARRSTRINGOUTPUT,
    options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_DRUCKERWORKER)

DESCRIPTOR.services_by_name['DruckerWorker'] = _DRUCKERWORKER

# @@protoc_insertion_point(module_scope)



#!/usr/bin/python
# -*- coding: utf-8 -*-


import json

from enum import Enum
from grpc import ServicerContext
from typing import Iterator, Union

from .logger import ServiceLoggerInterface
from .drucker_worker import PredictResult, Drucker
from .protobuf import drucker_pb2, drucker_pb2_grpc


DruckerInput = Union[
    drucker_pb2.StringInput, drucker_pb2.BytesInput,
    drucker_pb2.ArrIntInput, drucker_pb2.ArrFloatInput, drucker_pb2.ArrStringInput]
DruckerOutput = Union[
    drucker_pb2.StringOutput, drucker_pb2.BytesOutput,
    drucker_pb2.ArrIntOutput, drucker_pb2.ArrFloatOutput, drucker_pb2.ArrStringOutput]


class DruckerWorkerServicer(drucker_pb2_grpc.DruckerWorkerServicer):
    class Type(Enum):
        STRING = 1
        BYTES = 2
        ARRAY_INT = 3
        ARRAY_FLOAT = 4
        ARRAY_STRING = 5

    def __init__(self, logger: ServiceLoggerInterface, app: Drucker):
        self.logger = logger
        self.app = app

    def Process(self,
                request: DruckerInput,
                context: ServicerContext,
                response: DruckerOutput
                ) -> DruckerOutput:

        input = request.input
        try:
            ioption = json.loads(request.option.val)
        except:
            ioption = {request.option.val: request.option.val}

        single_output = self.app.get_type_output() in [self.Type.STRING, self.Type.BYTES]
        try:
            result = self.app.predict(input, ioption)
        except:
            if single_output:
                if isinstance(response, drucker_pb2.StringOutput):
                    label = "None"
                elif isinstance(response, drucker_pb2.BytesOutput):
                    label = b'None'
                else:
                    label = None
                result = PredictResult(label=label, score=0.0, option={})
            else:
                if isinstance(response, drucker_pb2.ArrStringOutput):
                    label = ["None"]
                elif isinstance(response, drucker_pb2.ArrIntOutput):
                    label = [0]
                elif isinstance(response, drucker_pb2.ArrFloatOutput):
                    label = [0.0]
                else:
                    label = None
                result = PredictResult(label=label, score=[0.0], option={})
        if single_output:
            response.output = result.label
            response.score = result.score
        else:
            response.output.extend(result.label)
            response.score.extend(result.score)
        response.option.val = result.option
        self.logger.emit(request, response, ioption.get('suppress_log_inout', False))
        return response

    def Predict_String_String(self,
                              request: drucker_pb2.StringInput,
                              context: ServicerContext
                              ) -> drucker_pb2.StringOutput:
        response = drucker_pb2.StringOutput()
        self.app.set_type(self.Type.STRING, self.Type.STRING)
        return self.Process(request, context, response)

    def Predict_String_Bytes(self,
                             request: drucker_pb2.StringInput,
                             context: ServicerContext
                             ) -> drucker_pb2.BytesOutput:
        response = drucker_pb2.BytesOutput()
        self.app.set_type(self.Type.STRING, self.Type.BYTES)
        yield self.Process(request, context, response)

    def Predict_String_ArrInt(self,
                              request: drucker_pb2.StringInput,
                              context: ServicerContext
                              ) -> drucker_pb2.ArrIntOutput:
        response = drucker_pb2.ArrIntOutput()
        self.app.set_type(self.Type.STRING, self.Type.ARRAY_INT)
        return self.Process(request, context, response)

    def Predict_String_ArrFloat(self,
                                request: drucker_pb2.StringInput,
                                context: ServicerContext
                                ) -> drucker_pb2.ArrFloatOutput:
        response = drucker_pb2.ArrFloatOutput()
        self.app.set_type(self.Type.STRING, self.Type.ARRAY_FLOAT)
        return self.Process(request, context, response)

    def Predict_String_ArrString(self,
                                 request: drucker_pb2.StringInput,
                                 context: ServicerContext
                                 ) -> drucker_pb2.ArrStringOutput:
        response = drucker_pb2.ArrStringOutput()
        self.app.set_type(self.Type.STRING, self.Type.ARRAY_STRING)
        return self.Process(request, context, response)

    def Predict_Bytes_String(self,
                             request_iterator: Iterator[drucker_pb2.BytesInput],
                             context: ServicerContext
                             ) -> drucker_pb2.StringOutput:
        for request in request_iterator:
            response = drucker_pb2.StringOutput()
            self.app.set_type(self.Type.BYTES, self.Type.STRING)
            return self.Process(request, context, response)

    def Predict_Bytes_Bytes(self,
                            request_iterator: Iterator[drucker_pb2.BytesInput],
                            context: ServicerContext
                            ) -> drucker_pb2.BytesOutput:
        for request in request_iterator:
            response = drucker_pb2.BytesOutput()
            self.app.set_type(self.Type.BYTES, self.Type.BYTES)
            yield self.Process(request, context, response)

    def Predict_Bytes_ArrInt(self,
                             request_iterator: Iterator[drucker_pb2.BytesInput],
                             context: ServicerContext
                             ) -> drucker_pb2.ArrIntOutput:
        for request in request_iterator:
            response = drucker_pb2.ArrIntOutput()
            self.app.set_type(self.Type.BYTES, self.Type.ARRAY_INT)
            return self.Process(request, context, response)

    def Predict_Bytes_ArrFloat(self,
                               request_iterator: Iterator[drucker_pb2.BytesInput],
                               context: ServicerContext
                               ) -> drucker_pb2.ArrFloatOutput:
        for request in request_iterator:
            response = drucker_pb2.ArrFloatOutput()
            self.app.set_type(self.Type.BYTES, self.Type.ARRAY_FLOAT)
            return self.Process(request, context, response)

    def Predict_Bytes_ArrString(self,
                                request_iterator: Iterator[drucker_pb2.BytesInput],
                                context: ServicerContext
                                ) -> drucker_pb2.ArrStringOutput:
        for request in request_iterator:
            response = drucker_pb2.ArrStringOutput()
            self.app.set_type(self.Type.BYTES, self.Type.ARRAY_STRING)
            return self.Process(request, context, response)

    def Predict_ArrInt_String(self,
                              request: drucker_pb2.ArrIntInput,
                              context: ServicerContext
                              ) -> drucker_pb2.StringOutput:
        response = drucker_pb2.StringOutput()
        self.app.set_type(self.Type.ARRAY_INT, self.Type.STRING)
        return self.Process(request, context, response)

    def Predict_ArrInt_Bytes(self,
                             request: drucker_pb2.ArrIntInput,
                             context: ServicerContext
                             ) -> drucker_pb2.BytesOutput:
        response = drucker_pb2.BytesOutput()
        self.app.set_type(self.Type.ARRAY_INT, self.Type.BYTES)
        yield self.Process(request, context, response)

    def Predict_ArrInt_ArrInt(self,
                              request: drucker_pb2.ArrIntInput,
                              context: ServicerContext
                              ) -> drucker_pb2.ArrIntOutput:
        response = drucker_pb2.ArrIntOutput()
        self.app.set_type(self.Type.ARRAY_INT, self.Type.ARRAY_INT)
        return self.Process(request, context, response)

    def Predict_ArrInt_ArrFloat(self,
                                request: drucker_pb2.ArrIntInput,
                                context: ServicerContext
                                ) -> drucker_pb2.ArrFloatOutput:
        response = drucker_pb2.ArrFloatOutput()
        self.app.set_type(self.Type.ARRAY_INT, self.Type.ARRAY_FLOAT)
        return self.Process(request, context, response)

    def Predict_ArrInt_ArrString(self,
                                 request: drucker_pb2.ArrIntInput,
                                 context: ServicerContext
                                 ) -> drucker_pb2.ArrStringOutput:
        response = drucker_pb2.ArrStringOutput()
        self.app.set_type(self.Type.ARRAY_INT, self.Type.ARRAY_STRING)
        return self.Process(request, context, response)

    def Predict_ArrFloat_String(self,
                                request: drucker_pb2.ArrFloatInput,
                                context: ServicerContext
                                ) -> drucker_pb2.StringOutput:
        response = drucker_pb2.StringOutput()
        self.app.set_type(self.Type.ARRAY_FLOAT, self.Type.STRING)
        return self.Process(request, context, response)

    def Predict_ArrFloat_Bytes(self,
                               request: drucker_pb2.ArrFloatInput,
                               context: ServicerContext
                               ) -> drucker_pb2.BytesOutput:
        response = drucker_pb2.BytesOutput()
        self.app.set_type(self.Type.ARRAY_FLOAT, self.Type.BYTES)
        yield self.Process(request, context, response)

    def Predict_ArrFloat_ArrInt(self,
                                request: drucker_pb2.ArrFloatInput,
                                context: ServicerContext
                                ) -> drucker_pb2.ArrIntOutput:
        response = drucker_pb2.ArrIntOutput()
        self.app.set_type(self.Type.ARRAY_FLOAT, self.Type.ARRAY_INT)
        return self.Process(request, context, response)

    def Predict_ArrFloat_ArrFloat(self,
                                  request: drucker_pb2.ArrFloatInput,
                                  context: ServicerContext
                                  ) -> drucker_pb2.ArrFloatOutput:
        response = drucker_pb2.ArrFloatOutput()
        self.app.set_type(self.Type.ARRAY_FLOAT, self.Type.ARRAY_FLOAT)
        return self.Process(request, context, response)

    def Predict_ArrFloat_ArrString(self,
                                   request: drucker_pb2.ArrFloatInput,
                                   context: ServicerContext
                                   ) -> drucker_pb2.ArrStringOutput:
        response = drucker_pb2.ArrStringOutput()
        self.app.set_type(self.Type.ARRAY_FLOAT, self.Type.ARRAY_STRING)
        return self.Process(request, context, response)

    def Predict_ArrString_String(self,
                                 request: drucker_pb2.ArrStringInput,
                                 context: ServicerContext
                                 ) -> drucker_pb2.StringOutput:
        response = drucker_pb2.StringOutput()
        self.app.set_type(self.Type.ARRAY_STRING, self.Type.STRING)
        return self.Process(request, context, response)

    def Predict_ArrString_Bytes(self,
                                request: drucker_pb2.ArrStringInput,
                                context: ServicerContext
                                ) -> drucker_pb2.BytesOutput:
        response = drucker_pb2.BytesOutput()
        self.app.set_type(self.Type.ARRAY_STRING, self.Type.BYTES)
        yield self.Process(request, context, response)

    def Predict_ArrString_ArrInt(self,
                                 request: drucker_pb2.ArrStringInput,
                                 context: ServicerContext
                                 ) -> drucker_pb2.ArrIntOutput:
        response = drucker_pb2.ArrIntOutput()
        self.app.set_type(self.Type.ARRAY_STRING, self.Type.ARRAY_INT)
        return self.Process(request, context, response)

    def Predict_ArrString_ArrFloat(self,
                                   request: drucker_pb2.ArrStringInput,
                                   context: ServicerContext
                                   ) -> drucker_pb2.ArrFloatOutput:
        response = drucker_pb2.ArrFloatOutput()
        self.app.set_type(self.Type.ARRAY_STRING, self.Type.ARRAY_FLOAT)
        return self.Process(request, context, response)

    def Predict_ArrString_ArrString(self,
                                    request: drucker_pb2.ArrStringInput,
                                    context: ServicerContext
                                    ) -> drucker_pb2.ArrStringOutput:
        response = drucker_pb2.ArrStringOutput()
        self.app.set_type(self.Type.ARRAY_STRING, self.Type.ARRAY_STRING)
        return self.Process(request, context, response)



# coding: utf-8


import sys
import os
import pathlib


root_path = pathlib.Path(os.path.abspath(__file__)).parent
sys.path.append(str(root_path))


from .drucker_pb2 import *
from .drucker_pb2_grpc import *



from sqlalchemy import (
    Column, String, Boolean, UniqueConstraint
)

from .dao import db


class ModelAssignment(db.ModelBase):
    __tablename__ = 'model_assignments'
    __table_args__ = (
        UniqueConstraint('service_name'),
        {'mysql_engine': 'InnoDB'}
    )

    service_name = Column(String(512), primary_key=True)
    model_path = Column(String(512), nullable=False)
    first_boot = Column(Boolean(), nullable=False)

    @property
    def serialize(self):
        return {
            'service_name': self.service_name,
            'model_path': self.model_path,
            'first_boot': self.first_boot
        }



#!/usr/bin/python
# -*- coding: utf-8 -*-

import traceback

import grpc
import types
import shutil
import uuid
import pickle
from pathlib import Path

from grpc import ServicerContext
from typing import Iterator

from .logger import SystemLoggerInterface
from .drucker_worker import Drucker, db, ModelAssignment
from .protobuf import drucker_pb2, drucker_pb2_grpc


def error_handling(error_response):
    """ Decorator for handling error

    Apply following processing on Servicer methods
    to handle errors.

    - DB transaction decorating for Servicer class.
      Confirm to call :func:``db.session.commit``
      on success operation and :func:``db.session.rollback``
      on fail.
    - Error setup for gRPC errors
    - Call :func:``on_error`` method (if defined) in the class
      to postprocess something on error

    Parameters
    ----------
    error_response
        gRPC response instance on error

    """

    def _wrapper_maker(func):
        def _wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as error:
                # DB rollback
                db.session.rollback()

                # gRPC
                context = args[2]
                context.set_code(grpc.StatusCode.UNKNOWN)
                context.set_details(str(error))

                servicer = args[0]
                if hasattr(servicer, 'on_error'):
                    assert isinstance(servicer.on_error, types.MethodType), \
                        'You must define on_error as method'
                    servicer.on_error(error)
                return error_response
            finally:
                db.session.close()

        return _wrapper

    return _wrapper_maker


class DruckerDashboardServicer(drucker_pb2_grpc.DruckerDashboardServicer):
    """ gRPC servicer to manage environment

    - Applications
        Machine leagning applications
    - Services
        Unit to deploy machine learning models
        (Corresponding to Service of K8S)
    - Models
        Machine learning model
    """

    # postfix for evaluate result
    EVALUATE_RESULT = '_eval_res.pkl'
    EVALUATE_DETAIL = '_eval_detail.pkl'

    def __init__(self, logger: SystemLoggerInterface, app: Drucker):
        self.logger = logger
        self.app = app

    def on_error(self, error: Exception):
        """ Postprocessing on error

        For detail, see :func:``on_error``

        Parameters
        ----------
        error : Exception
            Error to be handled
        """
        self.logger.error(str(error))
        self.logger.error(traceback.format_exc())

    def is_valid_upload_filename(self, filename: str) -> bool:
        if Path(filename).name == filename:
            return True
        return False

    def ServiceInfo(self,
                    request: drucker_pb2.ServiceInfoRequest,
                    context: ServicerContext
                    ) -> drucker_pb2.ServiceInfoResponse:
        """ Get service info.
        """
        return drucker_pb2.ServiceInfoResponse(
            application_name=self.app.config.APPLICATION_NAME,
            service_name=self.app.config.SERVICE_NAME,
            service_level=self.app.config.SERVICE_LEVEL_ENUM.value)

    @error_handling(drucker_pb2.ModelResponse(status=0, message='Error: Uploading model file.'))
    def UploadModel(self,
                    request_iterator: Iterator[drucker_pb2.UploadModelRequest],
                    context: ServicerContext
                    ) -> drucker_pb2.ModelResponse:
        """ Upload your latest ML model.
        """
        first_req = next(request_iterator)
        save_path = first_req.path
        if not self.is_valid_upload_filename(save_path):
            raise Exception(f'Error: Invalid model path specified -> {save_path}')

        tmp_path = self.app.get_model_path(uuid.uuid4().hex)
        Path(tmp_path).parent.mkdir(parents=True, exist_ok=True)
        with open(tmp_path, 'wb') as f:
            f.write(first_req.data)
            for request in request_iterator:
                f.write(request.data)
            del first_req
            f.close()

        model_path = self.app.get_model_path(save_path)
        Path(model_path).parent.mkdir(parents=True, exist_ok=True)
        shutil.move(tmp_path, model_path)
        return drucker_pb2.ModelResponse(status=1,
                                         message='Success: Uploading model file.')

    @error_handling(drucker_pb2.ModelResponse(status=0, message='Error: Switching model file.'))
    def SwitchModel(self,
                    request: drucker_pb2.SwitchModelRequest,
                    context: ServicerContext
                    ) -> drucker_pb2.ModelResponse:
        """ Switch your ML model to run.
        """
        if not self.is_valid_upload_filename(request.path):
            raise Exception(f'Error: Invalid model path specified -> {request.path}')

        model_assignment = self.app.db.session.query(ModelAssignment).filter(ModelAssignment.service_name == self.app.config.SERVICE_NAME).one()
        model_assignment.model_path = request.path
        model_assignment.first_boot = False
        self.app.db.session.commit()

        # :TODO: Use enum for SERVICE_INFRA
        if self.app.config.SERVICE_INFRA == "kubernetes":
            pass
        elif self.app.config.SERVICE_INFRA == "default":
            self.app.model_path = self.app.get_model_path()
            self.app.load_model()

        return drucker_pb2.ModelResponse(status=1,
                                         message='Success: Switching model file.')

    @error_handling(drucker_pb2.EvaluateModelResponse(metrics=drucker_pb2.EvaluationMetrics()))
    def EvaluateModel(self,
                      request_iterator: Iterator[drucker_pb2.EvaluateModelRequest],
                      context: ServicerContext
                      ) -> drucker_pb2.EvaluateModelResponse:
        """ Evaluate your ML model and save result.
        """
        first_req = next(request_iterator)
        data_path = first_req.data_path
        result_path = first_req.result_path
        if not self.is_valid_upload_filename(data_path):
            raise Exception(f'Error: Invalid evaluation file path specified -> {data_path}')
        if not self.is_valid_upload_filename(result_path):
            raise Exception(f'Error: Invalid evaluation result file path specified -> {result_path}')

        result, details = self.app.evaluate(self.app.get_eval_path(data_path))
        metrics = drucker_pb2.EvaluationMetrics(num=result.num,
                                                accuracy=result.accuracy,
                                                precision=result.precision,
                                                recall=result.recall,
                                                fvalue=result.fvalue,
                                                option=result.option)

        eval_result_path = self.app.get_eval_path(result_path)
        Path(eval_result_path).parent.mkdir(parents=True, exist_ok=True)
        with open(eval_result_path + self.EVALUATE_RESULT, 'wb') as f:
            pickle.dump(result, f)
        with open(eval_result_path + self.EVALUATE_DETAIL, 'wb') as f:
            pickle.dump(details, f)

        return drucker_pb2.EvaluateModelResponse(metrics=metrics)

    @error_handling(drucker_pb2.UploadEvaluationDataResponse(status=0, message='Error: Uploading evaluation data.'))
    def UploadEvaluationData(self,
                             request_iterator: Iterator[drucker_pb2.UploadEvaluationDataRequest],
                             context: ServicerContext
                             ) -> drucker_pb2.UploadEvaluationDataResponse:
        """ Save evaluation data
        """
        first_req = next(request_iterator)
        save_path = first_req.data_path
        if not self.is_valid_upload_filename(save_path):
            raise Exception(f'Error: Invalid evaluation file path specified -> {save_path}')

        eval_data = b''.join([first_req.data] + [r.data for r in request_iterator])
        eval_path = self.app.get_eval_path(save_path)
        Path(eval_path).parent.mkdir(parents=True, exist_ok=True)
        with open(eval_path, 'wb') as f:
            f.write(eval_data)

        return drucker_pb2.UploadEvaluationDataResponse(status=1,
                                                        message='Success: Uploading evaluation data.')



# Copyright 2018 The Drucker Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup

VERSION = "0.4.3"
PACKAGE_NAME = "rekcurd"
DEVELOPMENT_STATUS = "3 - Alpha"

# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

EXTRAS = {}
REQUIRES = []
with open('requirements.txt') as f:
    for line in f:
        line, _, _ = line.partition('#')
        line = line.strip()
        if ';' in line:
            requirement, _, specifier = line.partition(';')
            for_specifier = EXTRAS.setdefault(':{}'.format(specifier), [])
            for_specifier.append(requirement)
        else:
            REQUIRES.append(line)

with open('test-requirements.txt') as f:
    TESTS_REQUIRES = f.readlines()

with open("README.md", "r") as fh:
    LONG_DESCRIPTION = fh.read()

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description="A Python gRPC framework for serving a machine learning module written in Python.",
    author_email="",
    author="Drucker",
    license="Apache License Version 2.0",
    url="https://github.com/drucker/drucker",
    keywords=["Drucker", "Kubernetes"],
    install_requires=REQUIRES,
    tests_require=TESTS_REQUIRES,
    extras_require=EXTRAS,
    packages=['drucker', 'drucker.logger', 'drucker.utils',
              'drucker.models', 'drucker.protobuf'],
    include_package_data=True,
    long_description=LONG_DESCRIPTION,
    classifiers=[
        "Development Status :: %s" % DEVELOPMENT_STATUS,
        "Environment :: Web Environment",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        ],
)



# coding: utf-8


from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, scoped_session

from drucker.utils import DruckerConfig


class DruckerDAO(object):
    """ Data Access Object

    This implementation is inspired by Flask-SQLAlchemy's one.
    """

    def __init__(self, config: DruckerConfig = None):
        """
        Constructor
        :param config:
        """
        self.config = config
        self.engine = create_engine(
            'sqlite:///:memory:',
            encoding='utf-8',
            echo=True
        )
        self.session = scoped_session(
            sessionmaker(
                autocommit=False,
                autoflush=False,
                bind=self.engine
            )
        )
        self.ModelBase = declarative_base()
        if config is not None:
            self.init_app(config)

    def init_app(self, config: DruckerConfig):
        self.config = config
        self.engine = create_engine(
            self.__db_url(config),
            encoding='utf-8',
            echo=True
        )
        self.session = scoped_session(
            sessionmaker(
                autocommit=False,
                autoflush=False,
                bind=self.engine
            )
        )

    def __db_url(self, config: DruckerConfig):
        """ Get full URL for DB

        :TODO: Make configuarable of template of URL and encoding
        :TODO: Use Enum for DB_MODE
        :TODO: Use an appropriate "Exception"
        """
        if config.DB_MODE == "sqlite":
            db_name = "drucker.test.sqlite3" if config.TEST_MODE else "drucker.sqlite3"
            url = f'sqlite:///{db_name}'
        elif config.DB_MODE == "mysql":
            host = config.DB_MYSQL_HOST
            port = config.DB_MYSQL_PORT
            db_name = "test_"+config.DB_MYSQL_DBNAME if config.TEST_MODE else config.DB_MYSQL_DBNAME
            user = config.DB_MYSQL_USER
            password = config.DB_MYSQL_PASSWORD
            url = f'mysql+pymysql://{user}:{password}@{host}:{port}/{db_name}?charset=utf8'
        else:
            raise Exception("Invalid DB_MODE.")
        return url


db = DruckerDAO()
